#########################################
# Created 02/2020 Francesc Roura Adserias
#########################################
#-------------------------------------------------------------------
# Libraries & functions load
#-------------------------------------------------------------------
variable <- "tas"
fcst.sdate <- "202002"
skill <- TRUE
region <- TRUE
country <- FALSE
global <- TRUE
path.outside <- "/home/Earth/froura/"

cat("Variable:   "); print(variable) 
cat("Sdate: "); print(fcst.sdate)
cat("Skill: "); print(skill) 
cat("Country: "); print(country) 
cat("GLOBAL: "); print(global)
cat("Outpath: "); print(path.outside)

.libPaths(new = '/gpfs/projects/bsc32/share/R_libs/3.5') #va a buscar el path on es trobes aquests paquets

library(s2dverification) 
library(ncdf4)
library(abind)
library(easyVerification)
library(multiApply)
library(startR)
library(easyNCDF)
library(pryr) # To check mem usage.

setwd("/home/Earth/froura/repositories/S2S4E-backend-BSC/data-analysis/")
source("create_nc.R")  # Load function create_nc
source("create_nc_country.R")  # Load function create_nc
source("Calibration_fcst.R")
source("s2s4e_tools.R")
library(CSTools)

#-------------------------------------------------------------------
# Settings 
#-------------------------------------------------------------------

variable <- "tas" # for Decathlon we will only forecast temperature  
system.name <- "system5"
exp.dir <- "/esarchive/exp/ecmwf/system5c3s/" 
remap.method <- "con" 
hcst.inityear <- 2016 #1993 
hcst.endyear <- 2017 #2017
#ncores=16 #(?)
ncores = 1
split_factor=1 #(?)

lons.min <- 0  #0
lons.max <- 360  #360
lats.min <- -90  #-90
lats.max <- 90 # 90

#-------------------------------------------------------------------
# Init 
#-------------------------------------------------------------------

# frequencies of obs and exp for temperature (tas)

freq.exp <- "f6h"
freq.obs <- "f1h"

# Start dates and Lead times

fcst.month <- substr(fcst.sdate,5,6) 
fcst.year <- substr(fcst.sdate,1,4) 

fcst.sdate <- paste(fcst.year, fcst.month, sep = "")
hcst.sdates <- paste(strtoi(hcst.inityear):strtoi(hcst.endyear),
                     fcst.month, sep = "")

obs.sdates <- hcst.sdates

if (system.name == "system5") {
  hcst.nmember <- 25
} else {
  stop("The system is not system5") 
}   

ltmin <- 2
fcst.freq <- "monthly_mean"
ltmax <- 4

#-------------------------------------------------------------------
# Timeseries load 
#-------------------------------------------------------------------
obs.path <- paste0(
            "/esarchive/recon/ecmwf/era5/",fcst.freq,
            "/tas_",
            freq.obs,
            "/tas_$file_date$.nc"
            )

fcst.path <- paste0( 
             exp.dir,fcst.freq, "/tas_", freq.exp,
             "/tas_$sdate$01.nc"
             )

hcst.path <- paste0(
             exp.dir,fcst.freq,"/tas_",
             freq.exp,
             "/tas_$sdate$01.nc"
             )

#-------------------------------------------------------------------
# Load data
#-------------------------------------------------------------------

hcst <- Start(dat = hcst.path,
              var = variable,
              sdate = hcst.sdates,
              time = indices(ltmin:ltmax),
              latitude = values(list(lats.min, lats.max)),
              longitude = values(list(lons.min, lons.max)),
              synonims = list(latitude=c('lat','latitude'),
                              longitude=c('lon','longitude')), 
              ensemble = indices(1:hcst.nmember),
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = c('sdate')),
              retrieve = T)

fcst <- Start(dat = fcst.path, #les latituds es mantenen a 1:181, encar que les restringeixi a la linia 62-63. (?). Les longituds són les que li dic
              var = variable,
              sdate = fcst.sdate,
              time = indices(ltmin:ltmax),
              latitude = values(list(lats.min, lats.max)),
              longitude = values(list(lons.min, lons.max)),
              synonims = list(latitude=c('lat','latitude'),
                              longitude=c('lon','longitude')), 
              ensemble= 'all',
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = c('sdate')),
              retrieve = T)

dates <- attr(hcst, 'Variables')$common$time
dates_file <- sapply(dates, format, '%Y%m')
dim(dates_file) <- dim(dates)

obs <- Start(dat = obs.path, # hi ha una dimansió que diu 1:51, i no sé a què es refereix
             var = variable,
             file_date = dates_file,
             latitude = values(list(lats.min, lats.max)),
             longitude = values(list(lons.min, lons.max)),
             synonims = list(latitude=c('lat','latitude'),
                             longitude=c('lon','longitude')), 
             transform = CDORemapper,
             transform_params = list(
               grid = paste0(
                 "/esarchive/exp/ecmwf/system5c3s/",fcst.freq,
                 "/",variable,"_",
                 freq.exp,
                 ## TODO: replace hardcoded grid sample
                 #"/",variable,"_",fcst.sdate,"01.nc"
                 "/",variable,"_","201805","01.nc"
               ),
               method = 'con',
               crop = c(lons.min,lons.max,lats.min,lats.max)
             ), 
             transform_vars = c('latitude', 'longitude'), 
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = 'file_date'),
             split_multiselected_dims = TRUE,
             retrieve = TRUE)

print("######### DATA LOADING COMPLETED ##############")

#-------------------------------------------------------------------
# seasonal aggregation DECATHLON
#-------------------------------------------------------------------
seasonal.agg=F
if(seasonal.agg == T){
  # Això no funciona ara. He de fer la mitjana sense perdre els atributs dels elements obs, fcst i hcst.
  
  attr.hcst <- attributes(hcst) 
  attr.hcst$dim[4] <- 1
  hcst <- Mean1Dim(hcst,4)
  default_dims <- c(dat=1, var=1, sdate = 1, time=1, latitude=1, longitude=1, ensemble = 1)
  default_dims[names(dim(hcst))] <- dim(hcst)
  dim(hcst) <- default_dims
  attributes(hcst) <- attr.hcst
  
  attr.fcst <- attributes(fcst) 
  attr.fcst$dim[4] <- 1
  fcst <- Mean1Dim(fcst,4)
  default_dims <- c(dat=1, var=1, sdate = 1, time=1, latitude=1, longitude=1, ensemble = 1)
  default_dims[names(dim(fcst))] <- dim(fcst)
  dim(fcst) <- default_dims
  attributes(fcst) <- attr.fcst
  
  obs <- Mean1Dim(obs,4,narm = TRUE)
}

#-------------------------------------------------------------------
# Forecast Bias adjustment (calibration method)
#-------------------------------------------------------------------

default_dims <- c(dat=1, var=1, sdate = 1, time=1, latitude=1, longitude=1, ensemble = 1)
default_dims[names(dim(obs))] <- dim(obs)
dim(obs) <- default_dims

sub_obs<-Subset(obs,c('dat','var'),list(1,1),drop='selected')
sub_hcst<-Subset(hcst,c('dat','var'),list(1,1),drop='selected')
sub_fcst<-Subset(fcst,c('dat','var'),list(1,1),drop='selected')

lon <- attr(obs, 'Variab  les')$dat1$longitude
lat <- attr(obs, 'Variables')$dat1$latitude

fcst.times <- attr(fcst, 'Variables')$common$time
fcst.times <- sort(unique(sapply(as.character(fcst.times),
                                 substr, 1, 10)))
fcst.times[which(substr(fcst.times, 9, 10) == '01')] <- as.character(as.Date(fcst.times[which(substr(fcst.times, 9, 10) == '01')], "%Y-%m-%d")-1)


hcst.times <- attr(hcst, 'Variables')$common$time
hcst.times <- sort(unique(sapply(as.character(hcst.times),
                                 substr, 1, 10)))
hcst.times[which(substr(hcst.times, 9, 10) == '01')] <- as.character(as.Date(hcst.times[which(substr(hcst.times, 9, 10) == '01')], "%Y-%m-%d")-1)

remove(obs,hcst,fcst) # obs (?) el suprimeixo però sequeix apareixent-me com a no null

## TODO: Add accum vars 
if (variable == 'prlr' || variable == 'rsds'){
  sub_obs[sub_obs < 0 ] <- 0
  sub_hcst[sub_hcst < 0 ] <- 0
}

################# variables out ##################

mask.path <- '/home/Earth/froura/repositories/auto-s2s/data-analysis/masks/mask_region_decathlon.Rdata'
# var.exp
# var.obs

################# a partir d'aquí va a l'script general (s2s4e.global.R) ###################

